import 'package:flutter/material.dart';
import 'package:ytx_ui/src/progress_content_helper.dart';

typedef EmptyClickAction = void Function(ProgressContentState state);
typedef ErrorClickAction = void Function(ProgressContentState state);

class ProgressContent extends StatefulWidget {

  final Widget child;
  final Widget emptyWidget;
  final Widget errorWidget;
  final Widget progressWidget;
  final EmptyClickAction emptyClick;
  final ErrorClickAction errorClick;
  final String tag;
  final bool firstShowLoading;

  ProgressContent({
    Key key,
    this.child,
    this.emptyWidget,
    this.errorWidget,
    this.progressWidget,
    this.emptyClick,
    this.errorClick,
    this.tag,
    this.firstShowLoading = true,
  }): super(key: key);

  @override
  ProgressContentState createState() => ProgressContentState();
}

class ProgressContentState extends State<ProgressContent> {
  Widget _emptyDefaultWidget;
  Widget _progressDefaultWidget;
  Widget _errorDefaultWidget;
  ProgressContentWidgetProxy _contentWidgetProxy;

  bool _progressFlag = true;
  bool _emptyFlag = false;
  bool _errorFlag = false;
  bool _contentFlag = false;

  Widget _getEmptyWidget() {
    if (widget.emptyWidget != null) {
      return widget.emptyWidget;
    }
    if (_emptyDefaultWidget == null) {
      _emptyDefaultWidget = _contentWidgetProxy.createEmptyWidget(this, widget.emptyClick);
    }
    return _emptyDefaultWidget;
  }

  Widget _getProgressWidget() {
    if (widget.progressWidget != null) {
      return widget.progressWidget;
    }
    if (_progressDefaultWidget == null) {
      _progressDefaultWidget = _contentWidgetProxy.createProgressWidget(this);
    }
    return _progressDefaultWidget;
  }

  Widget _getErrorWidget() {
    if (widget.errorWidget != null) {
      return widget.errorWidget;
    }
    if (_errorDefaultWidget == null) {
      _errorDefaultWidget = _contentWidgetProxy.createErrorWidget(this, widget.errorClick);
    }
    return _errorDefaultWidget;
  }

  void showProgress() {
    if (_emptyFlag == false && _errorFlag == false && _progressFlag == true) {
      return ;
    }
    _emptyFlag = false;
    _errorFlag = false;
    _progressFlag = true;
    setState(() {});
  }

  void justShowProgress() {
    if (_emptyFlag == false && _errorFlag == false && _contentFlag == false && _progressFlag == true) {
      return ;
    }
    _emptyFlag = false;
    _errorFlag = false;
    _contentFlag = false;
    _progressFlag = true;
    setState(() {});
  }

  void showError() {
    if (_emptyFlag == false && _errorFlag == true && _contentFlag == false && _progressFlag == false) {
      return ;
    }
    _emptyFlag = false;
    _contentFlag = false;
    _progressFlag = false;
    _errorFlag = true;
    setState(() {});
  }

  void showEmpty() {
    if (_emptyFlag == true && _errorFlag == false && _contentFlag == false && _progressFlag == false) {
      return ;
    }
    _contentFlag = false;
    _progressFlag = false;
    _errorFlag = false;
    _emptyFlag = true;
    setState(() {});
  }

  void showContent() {
    if (_emptyFlag == false && _errorFlag == false && _contentFlag == true && _progressFlag == false) {
      return ;
    }
    _progressFlag = false;
    _errorFlag = false;
    _emptyFlag = false;
    _contentFlag = true;
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    if (widget.firstShowLoading) {
      _contentFlag = false;
      _errorFlag = false;
      _emptyFlag = false;
      _progressFlag = true;
    } else {
      _contentFlag = true;
      _errorFlag = false;
      _emptyFlag = false;
      _progressFlag = false;
    }
    _contentWidgetProxy = ProgressContentHelper().createProxy.call();
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> list = [];
    if (widget.child != null) {
      list.add(Opacity(
        child: widget.child,
        opacity: _contentFlag ? 1 : 0,
      ));
    }

    if (_emptyFlag) {
      Widget emptyWidget = _getEmptyWidget();
      if (emptyWidget != null) {
        list.add(emptyWidget);
      }
    }

    if (_errorFlag) {
      Widget errorWidget = _getErrorWidget();
      if (errorWidget != null) {
        list.add(errorWidget);
      }
    }

    if (_progressFlag) {
      Widget progressWidget = _getProgressWidget();
      if (progressWidget != null) {
        list.add(progressWidget);
      }
    }
    return Stack(
      children: list,
    );
  }
}

