import 'package:flutter/material.dart';
import 'package:ytx_ui/src/progress_content.dart';

typedef CreateProgressContentWidgetProxy = ProgressContentWidgetProxy Function();

class ProgressContentHelper {
  static ProgressContentHelper _instance;
  static ProgressContentHelper get instance => _getInstance();
  static ProgressContentHelper _getInstance() {
    if (_instance == null) {
      _instance = ProgressContentHelper._internal();
    }
    return _instance;
  }

  ProgressContentHelper._internal();
  factory ProgressContentHelper() {
    return _getInstance();
  }

  CreateProgressContentWidgetProxy createProxy = () {
    return DefaultProgressContentWidgetProxy();
  };

  void setWidgetProxy(CreateProgressContentWidgetProxy proxy) {
    this.createProxy = proxy;
  }
}

abstract class ProgressContentWidgetProxy {
  Widget createEmptyWidget(ProgressContentState state, EmptyClickAction emptyClickAction);
  Widget createProgressWidget(ProgressContentState state);
  Widget createErrorWidget(ProgressContentState state, ErrorClickAction errorClickAction);
}

class DefaultProgressContentWidgetProxy implements ProgressContentWidgetProxy {
  @override
  Widget createEmptyWidget(ProgressContentState state, EmptyClickAction emptyClickAction) {
    return Container(
      color: Colors.amberAccent,
      child: Center(
        child: GestureDetector(
          child: Text("暂无内容"),
          onTap: () => emptyClickAction(state),
        ),
      ),
    );
  }

  @override
  Widget createErrorWidget(ProgressContentState state, ErrorClickAction errorClickAction) {
    return Center(
      child: GestureDetector(
        child: Text("数据异常"),
        onTap: () => errorClickAction(state),
      ),
    );
  }

  @override
  Widget createProgressWidget(ProgressContentState state) {
    return Center(
      child: Text("正在加载中..."),
    );
  }
}
